/*
 *  arch-tag: Implemetation files for podcast download manager
 *
 *  Copyright (C) 2005 Renato Araujo Oliveira Filho - INdT <renato.filho@indt.org.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <libgnomevfs/gnome-vfs-uri.h>
#include <glib/gstdio.h>
#include <gtk/gtk.h>
#include <string.h>
#define __USE_XOPEN
#include <time.h>

#include "rb-preferences.h"
#include "eel-gconf-extensions.h"
#include "rb-podcast-download.h"
#include "rb-file-helpers.h"
#include "rb-debug.h"
#include "rb-podcast-marshal.h"
#include "rhythmdb.h"
#include "rhythmdb-query-model.h"
#include "rb-podcast-parse.h"
#include "rb-metadata.h"

#define BUFFER_SIZE 256
#define CONF_STATE_PODCAST_DOWNLOAD_HOUR        CONF_PREFIX "/state/podcast/download_hour"
#define CONF_STATE_PODCAST_DOWNLOAD_MINUTE      CONF_PREFIX "/state/podcast/download_minute"
#define CONF_STATE_PODCAST_DOWNLOAD_NEXT_TIME   CONF_PREFIX "/state/podcast/download_next_time"
#define CONF_STATE_PODCAST_SAVE_DIR		CONF_PREFIX "/state/podcast/save_dir"


struct RBPodcastDownloadPrivate
{
	RhythmDB *db;
	GList *download_list;
	guint source_sync;
	GMutex *mutex_job;
};

typedef struct 
{
	RBPodcastDownload *pd;
	RhythmDBEntry *entry;
	GnomeVFSAsyncHandle *read_handle;
	GnomeVFSURI *write_uri;
	GnomeVFSURI *read_uri;
	GMutex *mutex_working;
	
	guint total_size;
	guint progress;

	void 	(*impl_finish_job)	(RBPodcastDownload *pd);
}RBPodcastDownloadInfo;

enum 
{
	PROP_0,
	PROP_DB
};

enum
{
	STATUS_CHANGED,
	LAST_SIGNAL
};

static guint rb_podcast_download_signals[LAST_SIGNAL] = { 0 };
static GStaticMutex entry_add_mutex = G_STATIC_MUTEX_INIT;

static GObjectClass *parent_class = NULL;


/* functions */
static void rb_podcast_download_class_init 		(RBPodcastDownloadClass *klass);

static void rb_podcast_download_init 			(RBPodcastDownload *dp);

static GObject *rb_podcast_download_constructor 	(GType type, guint n_construct_properties,
			   				 GObjectConstructParam *construct_properties);

static void rb_podcast_download_finalize 		(GObject *object);

static void rb_podcast_download_set_property 		(GObject *object,
                                   			 guint prop_id,
		                                	 const GValue *value,
		                                	 GParamSpec *pspec);

static void rb_podcast_download_get_property 		(GObject *object,
							 guint prop_id,
		                                	 GValue *value,
                		                	 GParamSpec *pspec);

static void rb_podcast_download_copy_post 		(RBPodcastDownload *pd);

static int rb_podcast_download_mkdir_with_parents 	(const gchar *pathname,
					  		 int mode);

static gboolean rb_podcast_download_sync_head_cb 	(gpointer data);

static gboolean rb_podcast_download_head_query_cb 	(GtkTreeModel *query_model,
						   	 GtkTreePath *path, 
							 GtkTreeIter *iter,
						   	 RhythmDB *data);

static gboolean rb_podcast_download_save_metadata	(RhythmDB *db, 
						  	 RhythmDBEntry *entry, 
						  	 const char* uri);

static void rb_podcast_download_db_entry_added_cb 	(RBPodcastDownload *pd, 
							 RhythmDBEntry *entry);

static void rb_podcast_download_db_entry_deleted_cb 	(RBPodcastDownload *pd, 
							 RhythmDBEntry *entry);

static void rb_podcast_download_next_file 		(RBPodcastDownload * pd);

static const char* rb_podcast_download_save_img		(const char *img,
							 const char *dir_img);



/* async read file functions */
static guint download_progress_sync_cb			(GnomeVFSXferProgressInfo *info,
							 gpointer data);

static guint download_progress_update_cb		(GnomeVFSAsyncHandle *handle,
							 GnomeVFSXferProgressInfo *info,
							 gpointer data);

/* internal functions */
static void download_info_free				(RBPodcastDownloadInfo *data);
static RBPodcastDownloadInfo *download_info_new		(void);
static void start_job					(RBPodcastDownloadInfo *data);
static void end_job					(RBPodcastDownloadInfo *data);
static void cancel_all_job				(RBPodcastDownload *pd);
static void cancel_job					(RBPodcastDownloadInfo *pd);
static void write_job_data				(RBPodcastDownloadInfo *data);
static void finish_job_cb				(RBPodcastDownload *data);
static guint save_image_progress_cb 			(GnomeVFSXferProgressInfo *info, 
							 gpointer data);


GType
rb_podcast_download_get_type (void)
{
	static GType rb_podcast_download_type = 0;

	if (rb_podcast_download_type == 0)
	{
		static const GTypeInfo our_info =
		{
			sizeof (RBPodcastDownloadClass),
			NULL,
			NULL,
			(GClassInitFunc) rb_podcast_download_class_init,
			NULL,
			NULL,
			sizeof (RBPodcastDownload),
			0,
			(GInstanceInitFunc) rb_podcast_download_init
		};

		rb_podcast_download_type = g_type_register_static (G_TYPE_OBJECT,
							     "RBPodcastDownload",
							     &our_info, 0);
	}

	return rb_podcast_download_type;

}



static void
rb_podcast_download_class_init (RBPodcastDownloadClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);
	
	parent_class = g_type_class_peek_parent (klass);

	object_class->constructor = rb_podcast_download_constructor;	
	object_class->finalize = rb_podcast_download_finalize;

	object_class->set_property = rb_podcast_download_set_property;
        object_class->get_property = rb_podcast_download_get_property;

	 g_object_class_install_property (object_class,
                                          PROP_DB,
                                          g_param_spec_pointer ("db",
							       	"db",
							       	"database",
								G_PARAM_READWRITE));

	rb_podcast_download_signals[STATUS_CHANGED] = 
	       g_signal_new ("status_changed",
		       		G_OBJECT_CLASS_TYPE (object_class),
		 		GTK_RUN_LAST,
				G_STRUCT_OFFSET (RBPodcastDownloadClass, status_changed),
				NULL, NULL,
				rb_podcast_marshal_VOID__POINTER_ULONG,
				G_TYPE_NONE,
				2,
				G_TYPE_POINTER,
				G_TYPE_ULONG);
					
}	

static void
rb_podcast_download_init (RBPodcastDownload *pd)
{
	pd->priv = g_new0 (RBPodcastDownloadPrivate, 1);
	pd->priv->source_sync = 0;
	pd->priv->mutex_job = g_mutex_new();
	eel_gconf_monitor_add (CONF_PREFIX);
}

static GObject *
rb_podcast_download_constructor (GType type, guint n_construct_properties,
			   GObjectConstructParam *construct_properties)
{
	RBPodcastDownload *pd;

	pd = RB_PODCAST_DOWNLOAD (G_OBJECT_CLASS (parent_class)
					->constructor (type, n_construct_properties, construct_properties));
	
	return G_OBJECT (pd);

}

static void
rb_podcast_download_finalize (GObject *object)
{
	RBPodcastDownload *pd;
	g_return_if_fail (object != NULL);
        g_return_if_fail (RB_IS_PODCAST_DOWNLOAD (object));

	pd = RB_PODCAST_DOWNLOAD(object);
	
	g_return_if_fail (pd->priv != NULL);

	eel_gconf_monitor_remove (CONF_PREFIX);
	
	if (pd->priv->source_sync) {
		g_source_remove (pd->priv->source_sync);
		pd->priv->source_sync = 0;
	}

	cancel_all_job(pd);
	
	

	if (pd->priv->download_list)
		g_list_free (pd->priv->download_list);


	g_mutex_free (pd->priv->mutex_job);	
	
	g_static_mutex_free (&entry_add_mutex);
	
	g_free (pd->priv);
	
	G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void 
rb_podcast_download_set_property (GObject *object,
                           		guint prop_id,
	                                const GValue *value,
	                                GParamSpec *pspec)
{
	RBPodcastDownload *pd = RB_PODCAST_DOWNLOAD (object);

	switch (prop_id)
	{
	case PROP_DB:
		pd->priv->db = g_value_get_pointer (value);
	        g_signal_connect_object (G_OBJECT (pd->priv->db),
	                                 "entry-added",
	                                 G_CALLBACK (rb_podcast_download_db_entry_added_cb),
	                                 pd, G_CONNECT_SWAPPED);
		
	        g_signal_connect_object (G_OBJECT (pd->priv->db),
	                                 "entry_deleted",
	                                 G_CALLBACK (rb_podcast_download_db_entry_deleted_cb),
	                                 pd, G_CONNECT_SWAPPED);

		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
	}
}

static void 
rb_podcast_download_get_property (GObject *object,
					guint prop_id,
		                        GValue *value,
                		        GParamSpec *pspec)
{
	RBPodcastDownload *pd = RB_PODCAST_DOWNLOAD (object);

	switch (prop_id)
	{
	case PROP_DB:
		g_value_set_pointer(value, pd->priv->db);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
	}

	
}

RBPodcastDownload *
rb_podcast_download_new (RhythmDB *db)
{
	RBPodcastDownload *pd;

	pd = g_object_new (RB_TYPE_PODCAST_DOWNLOAD, "db", db, NULL);
	return pd; 
}


void
rb_podcast_download_add_entry (RBPodcastDownload *pd, RhythmDBEntry *entry)
{
	gulong status;
	if (entry == NULL)
		return;

	status = entry->podcast->status; 
	if ((status < 100) || (status == 102)) {
		RBPodcastDownloadInfo *data;
		
		if (status < 100) {
			GValue status_val = { 0, };
			g_value_init (&status_val, G_TYPE_ULONG);
			g_value_set_ulong (&status_val, 102);
			rhythmdb_entry_set (pd->priv->db, entry, RHYTHMDB_PROP_STATUS, &status_val);
			g_value_unset (&status_val);
		}
	
		rb_debug ("Try insert entry for download.");

		data  = download_info_new();
		data->pd = pd;	
		data->entry = entry;
		data->impl_finish_job = finish_job_cb;
		
        	g_static_mutex_lock (&entry_add_mutex);
		pd->priv->download_list =  g_list_append (pd->priv->download_list, data);
		g_static_mutex_unlock (&entry_add_mutex);
		
		rb_podcast_download_next_file (pd);
	}
	
	
}


void
rb_podcast_download_start_sync(RBPodcastDownload *pd)
{
	gint next_time =  eel_gconf_get_integer(CONF_STATE_PODCAST_DOWNLOAD_NEXT_TIME);


	if (pd->priv->source_sync != 0) {
		g_source_remove (pd->priv->source_sync);
		pd->priv->source_sync = 0;
	}

	
	if (next_time > 0) {
		pd->priv->source_sync = g_timeout_add (next_time * 1000, (GSourceFunc ) rb_podcast_download_sync_head_cb, pd);
	}

}

static gboolean
rb_podcast_download_sync_head_cb (gpointer data)
{
	RBPodcastDownload *pd = (RBPodcastDownload *) data;
	
	GtkTreeModel* query_model = GTK_TREE_MODEL (rhythmdb_query_model_new_empty(pd->priv->db));	


	rhythmdb_do_full_query (pd->priv->db, query_model,
                                RHYTHMDB_QUERY_PROP_EQUALS,
                                RHYTHMDB_PROP_TYPE,
                                RHYTHMDB_ENTRY_TYPE_PODCAST_HEAD,
                                RHYTHMDB_QUERY_END);

 	gtk_tree_model_foreach (query_model,
		                (GtkTreeModelForeachFunc)rb_podcast_download_head_query_cb,
                                pd->priv->db);


	g_object_unref (query_model);
	rb_podcast_download_start_sync (pd); 

	return FALSE;
 
}
	
static gboolean
rb_podcast_download_head_query_cb (GtkTreeModel *query_model,
					GtkTreePath *path, GtkTreeIter *iter,
					RhythmDB *data)
{
        const char* uri;
        RhythmDBEntry* entry;

        gtk_tree_model_get (query_model, iter, 0, &entry, -1);
        uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);

	rb_podcast_download_add_url(data, uri);
	
        return FALSE;
}

static void
rb_podcast_download_next_file (RBPodcastDownload * pd)
{

	rb_debug ("try lock file_process mutex");
	if (g_mutex_trylock (pd->priv->mutex_job) == TRUE) {
		gint size;
		
		g_static_mutex_lock (&entry_add_mutex);
		size = g_list_length (pd->priv->download_list);
		g_static_mutex_unlock (&entry_add_mutex);
			
		if (size > 0)
			rb_podcast_download_copy_post (pd);
		else
			g_mutex_unlock (pd->priv->mutex_job);
	}
	else
		rb_debug ("not start");
}

static void 
rb_podcast_download_copy_post (RBPodcastDownload *pd)
{
	GnomeVFSURI *remote_uri = NULL;
	GnomeVFSURI *local_uri = NULL;
	
	GValue location_val = { 0, };
	
	const char *short_name;
	const char *local_file_name;
	const char *dir_name;
	const char *location; 
	const char *album_name; 
	const char *conf_dir_name;
	
	RBPodcastDownloadInfo *data = NULL; 
	
	RhythmDBEntry *entry;

	rb_debug ("Stating copy file");
	g_value_init (&location_val, G_TYPE_STRING);
	conf_dir_name = eel_gconf_get_string (CONF_STATE_PODCAST_SAVE_DIR);
		
	//get first element of list
	g_static_mutex_lock (&entry_add_mutex);

	data = (RBPodcastDownloadInfo *) g_list_first(pd->priv->download_list)->data;
	if (data == NULL)
		return;
			
	g_static_mutex_unlock (&entry_add_mutex);

	
	entry = data->entry;
			
	g_assert (entry != NULL);
		
	location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
	album_name = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM);	
	
	rb_debug ("processing %s", location);
	
	remote_uri = gnome_vfs_uri_new (location);
	if (!remote_uri) {
		goto next_step;
	}
		
	if (gnome_vfs_uri_is_local (remote_uri)) {	
		goto next_step;
	}


	short_name = gnome_vfs_uri_extract_short_name (remote_uri);
		
	
	//verify if dir existis
	dir_name = g_build_filename (g_get_home_dir (),
	 			     (conf_dir_name == NULL ? "My Podcast" : conf_dir_name),
				     album_name,
				     NULL);
	

	 if (!g_file_test (dir_name, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
		if (rb_podcast_download_mkdir_with_parents (dir_name, 0750) != 0) {
			rb_debug ("Error on create local dirs.");
			goto next_step;
		}
	 }
		 		 
	local_file_name = g_build_filename (dir_name,
	 				    short_name,
					   NULL);	

	local_uri = gnome_vfs_uri_new (local_file_name);
	if (!local_uri) {
		goto next_step;
	}

	//TODO: suport resume file
	if (g_file_test (local_file_name, G_FILE_TEST_EXISTS)) {
		rb_debug ("Removing existent file.");
		g_remove (local_file_name);
	}

	
	
	data->read_uri = remote_uri;
	data->write_uri = local_uri;
	
	start_job (data);
	return;
	
next_step:
	
	if (remote_uri)
		gnome_vfs_uri_unref (remote_uri);

	if (local_uri)
		gnome_vfs_uri_unref (local_uri);

	g_static_mutex_lock (&entry_add_mutex);
	pd->priv->download_list = g_list_remove(pd->priv->download_list, (gconstpointer ) data);
	g_static_mutex_unlock (&entry_add_mutex);

	download_info_free (data);

	g_mutex_unlock (pd->priv->mutex_job);
	rb_podcast_download_next_file (pd);
}


static int
rb_podcast_download_mkdir_with_parents (const gchar *pathname,
		   			int          mode)
{
  gchar *fn, *p;

  if (pathname == NULL || *pathname == '\0')
    {
      return -1;
    }

  fn = g_strdup (pathname);

  if (g_path_is_absolute (fn))
    p = (gchar *) g_path_skip_root (fn);
  else
    p = fn;

  do
    {
      while (*p && !G_IS_DIR_SEPARATOR (*p))
	p++;
      
      if (!*p)
	p = NULL;
      else
	*p = '\0';
      
      if (!g_file_test (fn, G_FILE_TEST_EXISTS))
	{
	  if (g_mkdir (fn, mode) == -1)
	    {
	      g_free (fn);
	      return -1;
	    }
	}
      else if (!g_file_test (fn, G_FILE_TEST_IS_DIR))
	{
	  g_free (fn);
	  return -1;
	}
      if (p)
	{
	  *p++ = G_DIR_SEPARATOR;
	  while (*p && G_IS_DIR_SEPARATOR (*p))
	    p++;
	}
    }
  while (p);

  g_free (fn);

  return 0;
}

gboolean
rb_podcast_download_add_url (RhythmDB *db, const char* url)
{
	GValue description_val = { 0, };
	GValue title_val = { 0, };
	GValue subtitle_val = { 0, };
	GValue summary_val = { 0, };
	GValue lang_val = { 0, };
	GValue copyright_val = { 0, };
	GValue image_val = { 0, };
	GValue author_val = { 0, };
	gulong status;
	gchar *valid_url = gnome_vfs_make_uri_from_input (url);

	RhythmDBEntry *entry;

	GList *lst_songs;

	RBPodcastChannel *data = g_new0 (RBPodcastChannel, 1);
	

	rb_podcast_parse_load_feed (data, valid_url);	


	if (data->title	== NULL) {
		g_list_free (data->lst_itens);
		g_free (data);
		return FALSE;
	}

	//processing podcast head
	entry = rhythmdb_entry_lookup_by_location (db, valid_url);
	if (entry) {
		rb_debug ("Head found");
		goto load_posts;
	}
	
	rb_debug ("Insert new entry");
	entry = rhythmdb_entry_new (db,
				    RHYTHMDB_ENTRY_TYPE_PODCAST_HEAD,
			    	    valid_url);


	rb_debug("New entry create\n");
	
	g_value_init (&title_val, G_TYPE_STRING);
	g_value_set_string (&title_val, (gchar* ) data->title);
	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_TITLE, &title_val);
	g_value_unset (&title_val);

	if (!data->author)
		data->author = xmlCharStrdup("<unknowm>");
	
	g_value_init (&author_val, G_TYPE_STRING);
	g_value_set_string (&author_val, (gchar* ) data->author);
	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ARTIST, &author_val);
	g_value_unset (&author_val);
	
	
	if (data->subtitle) {
		g_value_init (&subtitle_val, G_TYPE_STRING);
		g_value_set_string (&subtitle_val, (gchar* ) data->subtitle);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_SUBTITLE, &subtitle_val);
		g_value_unset (&subtitle_val);
	}
	
	if (data->description) {
		g_value_init (&description_val, G_TYPE_STRING);
		g_value_set_string (&description_val, (gchar* ) data->description);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_DESCRIPTION, &description_val);
		g_value_unset (&description_val);
	}
	
	if (data->summary) {
		g_value_init (&summary_val, G_TYPE_STRING);
		g_value_set_string (&summary_val, (gchar* ) data->summary);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_SUMMARY, &summary_val);
		g_value_unset (&summary_val);
	}

	if (data->lang) {
		g_value_init (&lang_val, G_TYPE_STRING);
		g_value_set_string (&lang_val, (gchar* ) data->lang);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_LANG, &lang_val);
		g_value_unset (&lang_val);
	}

	if (data->copyright) { 
		g_value_init (&copyright_val, G_TYPE_STRING);
		g_value_set_string (&copyright_val, (gchar* ) data->copyright);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_COPYRIGHT, &copyright_val);
		g_value_unset (&copyright_val);
	}

	if (data->img) {
		const char *image_file =  rb_podcast_download_save_img ((gchar* ) data->img, (gchar* ) data->title);
		g_value_init (&image_val, G_TYPE_STRING);
		g_value_set_string (&image_val, image_file);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_IMAGE, &image_val);
		g_value_unset (&image_val);
	}

	rhythmdb_commit (db);
	
	rb_debug("Podcast head Inserted");
	
load_posts:
	
	lst_songs = data->lst_itens;
	status = 102;
	while (lst_songs) {
		RBPodcastItem *item = (RBPodcastItem *) lst_songs->data;
		if (item) {
			rb_podcast_download_add_post (db, 
						(gchar* ) data->title,
						(gchar* ) item->title, 
						valid_url,
						(gchar* ) data->author,
						(gchar* ) item->url,
						status,
						(gulong ) item->pub_date);
			status = 103;
		}
		lst_songs = lst_songs->next;
	}
	
	g_list_free (data->lst_itens);
	return TRUE;
}

void
rb_podcast_download_add_post (RhythmDB *db,
			      const char *name,
			      const char *title,
			      const char *subtitle,
			      const char *generator,
			      const char *uri,
			      gulong status,
			      gulong date)
{
	//GThread* copy_thread = NULL;
	GValue title_val = { 0, };
	GValue artist_val = { 0, };
	GValue album_val = { 0, };
	GValue mountpoit_val = { 0, };
	GValue status_val = { 0, };
	GValue time_val = { 0, };
	GValue genre_val = { 0, };
	GValue subtitle_val = { 0, };
	GValue value =  { 0, };
	GTimeVal time;
	
	RhythmDBEntry *entry;

	if (uri && name && title && date && g_utf8_validate(uri, -1, NULL)) {
		entry = rhythmdb_entry_lookup_by_location (db, uri);

		if (entry)
			return;

		g_value_init (&album_val, G_TYPE_STRING);
		g_value_set_string (&album_val, name);
		
		g_value_init (&mountpoit_val, G_TYPE_STRING);
		g_value_set_string (&mountpoit_val, uri);
	
		g_value_init (&status_val, G_TYPE_ULONG);
		g_value_set_ulong (&status_val, status);

		g_value_init (&genre_val, G_TYPE_STRING);
		g_value_set_string (&genre_val, "Podcast");

		g_value_init (&time_val, G_TYPE_ULONG);
		g_value_set_ulong (&time_val, date);
		
		g_value_init (&title_val, G_TYPE_STRING);
		g_value_set_string (&title_val, title);

		g_value_init (&subtitle_val, G_TYPE_STRING);
		g_value_set_string (&subtitle_val, subtitle);

	
		g_value_init (&artist_val, G_TYPE_STRING);
		if (generator) 
			g_value_set_string (&artist_val, generator);
		else
			g_value_set_string (&artist_val, "");


		
		entry = rhythmdb_entry_new (db,
					    RHYTHMDB_ENTRY_TYPE_PODCAST_POST,
				    	    uri);
		
		/* initialize the last played date to 0=never */
		g_value_init (&value, G_TYPE_ULONG);
		g_value_set_ulong (&value, 0);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_LAST_PLAYED, &value);
		g_value_unset (&value);

		/* initialize the rating */
		g_value_init (&value, G_TYPE_DOUBLE);
		g_value_set_double (&value, 2.5);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_RATING, &value);
		g_value_unset (&value);

		/* first seen */
		g_get_current_time (&time);
		
		g_value_init (&value, G_TYPE_ULONG);
		g_value_set_ulong (&value, time.tv_sec);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_FIRST_SEEN, &value);
		g_value_unset (&value);

			
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_GENRE, &genre_val);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_POST_TIME, &time_val);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_TITLE, &title_val);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_SUBTITLE, &subtitle_val);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ARTIST, &artist_val);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ALBUM, &album_val);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_MOUNTPOINT, &mountpoit_val);
		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_STATUS, &status_val);
			
		rhythmdb_commit (db);
		
		g_value_unset (&time_val);
		g_value_unset (&title_val);
		g_value_unset (&subtitle_val);
		g_value_unset (&album_val);
		g_value_unset (&mountpoit_val);
		g_value_unset (&status_val);
		g_value_unset (&artist_val);

	}

}

static gboolean
rb_podcast_download_save_metadata(RhythmDB *db, RhythmDBEntry *entry, const char* uri)
{
	RBMetaData *md = rb_metadata_new();
	GError *error = NULL;
	GValue val = { 0, };
	const char *mime;
	
	rb_debug("Loading podcast metadata");
        rb_metadata_load (md, uri, &error);	

	mime = rb_metadata_get_mime (md);
	if (mime) {
		g_value_init (&val, G_TYPE_STRING);
		g_value_set_string (&val, mime);
		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_MIMETYPE, &val);
		g_value_unset (&val);
	}


	if (error != NULL) {
		g_object_unref (md);
		return FALSE;
	}

	if (rb_metadata_get (md,
			     RB_METADATA_FIELD_DURATION,
			     &val)) {
		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_DURATION, &val);
		g_value_unset (&val);
	}

	if (rb_metadata_get (md, 
			     RB_METADATA_FIELD_BITRATE,
			     &val)) {
		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_BITRATE, &val);
		g_value_unset (&val);
	}

	g_object_unref (md);
	return TRUE;
}

static void
rb_podcast_download_db_entry_added_cb (RBPodcastDownload *pd, RhythmDBEntry *entry)
{

	if (entry->type != RHYTHMDB_ENTRY_TYPE_PODCAST_POST)
		return;
	
        rb_podcast_download_add_entry(pd, entry);
}

static void 
write_job_data (RBPodcastDownloadInfo *data)
{

	GValue val = {0, };
	RhythmDB *db = data->pd->priv->db;

	rb_debug ("in the write_job");
	
	g_value_init (&val, G_TYPE_UINT64);
	g_value_set_uint64 (&val, data->total_size);
	rhythmdb_entry_set (db, data->entry, RHYTHMDB_PROP_FILE_SIZE, &val);
	g_value_unset (&val);
	
	
	g_value_init (&val, G_TYPE_ULONG);
	g_value_set_ulong (&val, 100);
	rhythmdb_entry_set (db, data->entry, RHYTHMDB_PROP_STATUS, &val);
	g_value_unset (&val);

	g_value_init (&val, G_TYPE_STRING);
	g_value_set_string (&val,  
			    gnome_vfs_uri_to_string (data->write_uri, GNOME_VFS_URI_HIDE_NONE));
	rhythmdb_entry_set (db, data->entry, RHYTHMDB_PROP_MOUNTPOINT, &val);
	g_value_unset (&val);
	
	rb_podcast_download_save_metadata (db, data->entry, 
		       			   gnome_vfs_uri_to_string (data->write_uri, GNOME_VFS_URI_HIDE_NONE));

	rhythmdb_commit (db);
}

static const char*
rb_podcast_download_save_img (const char *img, const char *dir_img)
{

	GdkPixbuf *img_pixbuf;
	GError *error;
	
	GnomeVFSResult ret;
	GnomeVFSURI *source_uri;
	GnomeVFSURI *target_uri;
	
	const char* source_name;
	const char* target_name;
	const char* short_name;
	const char* dir_name;
	
	const char* conf_dir_name = eel_gconf_get_string(CONF_STATE_PODCAST_SAVE_DIR);


	//verify if dir existis
	dir_name = g_build_filename (g_get_home_dir (),
	 			     (conf_dir_name == NULL ? "My Podcast" : conf_dir_name),
				     dir_img,
				     NULL);
		
	if (!g_file_test (dir_name, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
 		if (rb_podcast_download_mkdir_with_parents (dir_name, 0750) != 0) {
			rb_debug("Error on create local dirs.");
			return NULL;
		}
	}

	source_name =  gnome_vfs_make_uri_from_input (img);
	

	source_uri = gnome_vfs_uri_new (source_name);
	if (!source_uri) 
		return NULL;
	
	short_name = gnome_vfs_uri_extract_short_name(source_uri);

	target_name = g_build_filename (dir_name,
			       	    short_name,
				    NULL);	

	target_uri = gnome_vfs_uri_new (target_name);

	ret = gnome_vfs_xfer_uri (source_uri,
			    	  target_uri,
		    	    	  GNOME_VFS_XFER_DEFAULT,
			          GNOME_VFS_XFER_ERROR_MODE_ABORT,
			          GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
			          (GnomeVFSXferProgressCallback ) save_image_progress_cb,
			          NULL);

	gnome_vfs_uri_unref (target_uri);
	gnome_vfs_uri_unref (source_uri);

	error = NULL;
	
	img_pixbuf = gdk_pixbuf_new_from_file_at_scale(target_name, 200, 200, TRUE, &error);
	if (error != NULL)
		return NULL;
	
	g_remove (target_name);
	target_name =  g_build_filename (dir_name, "image.jpg", NULL);
	
	if (gdk_pixbuf_save (img_pixbuf, target_name, "jpeg", &error, NULL) != TRUE) {
		gdk_pixbuf_unref (img_pixbuf);
		return NULL;
	}

	gdk_pixbuf_unref (img_pixbuf);
	
	return target_name;
}

guint
save_image_progress_cb (GnomeVFSXferProgressInfo *info, gpointer data)
{
	return GNOME_VFS_XFER_ERROR_ACTION_SKIP;
}

static void
download_info_free (RBPodcastDownloadInfo *data)
{
	if (data->write_uri) {
		gnome_vfs_uri_unref (data->write_uri);
		data->write_uri = NULL;
	}

	if (data->read_uri) {
		gnome_vfs_uri_unref (data->read_uri);
		data->read_uri = NULL;
	}

	
	g_mutex_free (data->mutex_working);

	g_free (data);
}

static RBPodcastDownloadInfo*
download_info_new (void)
{
	RBPodcastDownloadInfo *data = g_new0 (RBPodcastDownloadInfo, 1);
	data->pd = NULL;
	data->entry = NULL;
	data->write_uri = NULL;
	data->read_uri = NULL;
	data->mutex_working = g_mutex_new ();
	data->total_size = 0;
	data->progress = 0;	
	data->impl_finish_job = NULL;

	return data;
}

static void 
start_job (RBPodcastDownloadInfo *data)
{
	
	GList *source_uri_list = NULL;
	GList *target_uri_list = NULL;

	rb_debug ("start job");

	source_uri_list = g_list_append (source_uri_list, data->read_uri);
	target_uri_list = g_list_append (target_uri_list, data->write_uri);

	g_mutex_lock (data->mutex_working);

	gnome_vfs_async_xfer ( &data->read_handle,
				source_uri_list,
				target_uri_list,
				GNOME_VFS_XFER_DEFAULT,
				GNOME_VFS_XFER_ERROR_MODE_QUERY,
				GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
				GNOME_VFS_PRIORITY_DEFAULT,
				(GnomeVFSAsyncXferProgressCallback ) download_progress_update_cb,
				data,
				(GnomeVFSXferProgressCallback ) download_progress_sync_cb,
				data);
}

static void 
cancel_all_job	(RBPodcastDownload *pd)
{
	rb_debug ("cancel all job");
	while (g_list_length (pd->priv->download_list) > 0) 
	{
		RBPodcastDownloadInfo *data = (RBPodcastDownloadInfo *) g_list_first (pd->priv->download_list)->data;
		cancel_job (data);				
	}
}

static void 
end_job	(RBPodcastDownloadInfo *data)
{
	rb_debug ("end_job");
	g_static_mutex_lock (&entry_add_mutex);
	data->pd->priv->download_list = g_list_remove (data->pd->priv->download_list, data);
	g_static_mutex_unlock (&entry_add_mutex);
	
	g_mutex_unlock (data->mutex_working);
	g_mutex_unlock (data->pd->priv->mutex_job);
	data->impl_finish_job (data->pd);

	download_info_free (data);

}

static void
cancel_job (RBPodcastDownloadInfo *data)
{
	if (g_mutex_trylock (data->mutex_working) == FALSE) {
		rb_debug ("cancel %p job", data);
		gnome_vfs_async_cancel (data->read_handle);
		g_mutex_unlock (data->mutex_working);
		g_mutex_unlock (data->pd->priv->mutex_job);
	}
	else {
		g_mutex_unlock (data->mutex_working);
	}

	g_static_mutex_lock (&entry_add_mutex);
	data->pd->priv->download_list = g_list_remove (data->pd->priv->download_list, data);
	g_static_mutex_unlock (&entry_add_mutex);

	download_info_free (data);
}

static void
finish_job_cb (RBPodcastDownload *pd)
{
	rb_debug ("finish_job");
	rb_podcast_download_next_file  (pd);
}

static guint 
download_progress_sync_cb (GnomeVFSXferProgressInfo *info, gpointer cb_data)
{
	RBPodcastDownloadInfo *data = (RBPodcastDownloadInfo *) cb_data;

	if (info->status != GNOME_VFS_XFER_PROGRESS_STATUS_OK) { 
		rb_debug ("error on download");
		data->entry->podcast->status = 101;
		end_job (data);
	}

	return GNOME_VFS_XFER_ERROR_ACTION_SKIP;

}

static guint 
download_progress_update_cb (GnomeVFSAsyncHandle *handle, GnomeVFSXferProgressInfo *info, gpointer cb_data)
{
	RBPodcastDownloadInfo *data = (RBPodcastDownloadInfo *) cb_data;
	guint local_progress  = 0;

	if (info->status != GNOME_VFS_XFER_PROGRESS_STATUS_OK) { 
		data->entry->podcast->status = 101;
		rb_debug ("error on download");
		end_job (data);
	}
	
	if (info->phase  == GNOME_VFS_XFER_PHASE_COMPLETED) {
		data->total_size = info->file_size;
		write_job_data (data);
		end_job (data);
	}

	if ((info->phase == GNOME_VFS_XFER_PHASE_COPYING) &&
	    (data->entry != NULL))	{
	
		local_progress = (gint) 100 * info->total_bytes_copied / info->file_size;

		if (local_progress != data->progress) {
			data->entry->podcast->status = local_progress;
			g_signal_emit (data->pd, rb_podcast_download_signals[STATUS_CHANGED],
				       0, data->entry, local_progress);
			data->progress = local_progress;
		}
	}
	
	return GNOME_VFS_XFER_ERROR_ACTION_SKIP;
}

gboolean
rb_podcast_download_remove_url (RhythmDB *db, const char* url)
{
	RhythmDBEntry *entry = rhythmdb_entry_lookup_by_location (db, url);
	if (entry) {
		g_remove (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_IMAGE));
		rhythmdb_entry_delete (db, entry);
		rhythmdb_commit (db);
		return TRUE;
	}
	return FALSE;
}

static void 
rb_podcast_download_db_entry_deleted_cb (RBPodcastDownload *pd, RhythmDBEntry *entry)
{
	const gchar *dir_name;
	const gchar *conf_dir_name;

	if (entry->type != RHYTHMDB_ENTRY_TYPE_PODCAST_POST)
		return;


	rb_debug ("removing file %s", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MOUNTPOINT));
	gnome_vfs_unlink (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MOUNTPOINT));
	

	
	//remove dir
	rb_debug ("removing dir");
	conf_dir_name = eel_gconf_get_string (CONF_STATE_PODCAST_SAVE_DIR);
	dir_name = g_build_filename (g_get_home_dir (),
				     (conf_dir_name == NULL ? "My Podcast" : conf_dir_name),
				     rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM),
				     NULL);
	gnome_vfs_remove_directory (dir_name);
	
		
}
